#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella button widgets
"""

from __future__ import print_function, division, absolute_import

import artella
from artella.core import qtutils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets


if not qtutils.QT_AVAILABLE:
    class ArtellaToolButton(object):
        def __init__(self, *args, **kwargs):
            super(ArtellaToolButton, self).__init__()
else:
    class ArtellaToolButton(QtWidgets.QToolButton):
        def __init__(self, parent=None):
            super(ArtellaToolButton, self).__init__(parent=parent)

            self._artella_image = None
            self._artella_size = artella.theme.default_size

            self.setAutoExclusive(False)
            self.setAutoRaise(True)
            self.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)

            self._polish_icon()
            self.toggled.connect(self._polish_icon)

        def enterEvent(self, event):
            if self._artella_image:
                self.setIcon(artella.ResourcesMgr().icon(self._artella_image, color=artella.theme.main_color))
            return super(ArtellaToolButton, self).enterEvent(event)

        def leaveEvent(self, event):
            self._polish_icon()
            return super(ArtellaToolButton, self).leaveEvent(event)

        def get_artella_size(self):
            return self._artella_size

        def set_artella_size(self, value):
            self._artella_size = value
            self.style().polish(self)
            if self.toolButtonStyle() == QtCore.Qt.ToolButtonIconOnly:
                self.setFixedSize(QtCore.QSize(self._artella_size, self._artella_size))

        def get_artella_image(self):
            return self._artella_image

        def set_artella_image(self, path):
            self._artella_image = path
            self._polish_icon()

        artella_size = QtCore.Property(int, get_artella_size, set_artella_size)

        def huge(self):
            self.set_artella_size(artella.theme.huge)
            return self

        def large(self):
            self.set_artella_size(artella.theme.large)
            return self

        def medium(self):
            self.set_artella_size(artella.theme.medium)
            return self

        def small(self):
            self.set_artella_size(artella.theme.small)
            return self

        def tiny(self):
            self.set_artella_size(artella.theme.tiny)
            return self

        def image(self, path):
            self.set_artella_image(path)
            return self

        def icon_only(self):
            self.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
            self.setFixedSize(QtCore.QSize(self._artella_size, self._artella_size))
            return self

        def text_only(self):
            self.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
            return self

        def text_beside_icon(self):
            self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
            return self

        def text_under_icon(self):
            self.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
            return self

        def _polish_icon(self, *args, **kwargs):
            if self._artella_image:
                if self.isCheckable() and self.isChecked():
                    self.setIcon(artella.ResourcesMgr().icon(self._artella_image, color=artella.theme.main_color))
                else:
                    self.setIcon(artella.ResourcesMgr().icon(self._artella_image))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella exceptions
"""

from __future__ import print_function, division, absolute_import

import logging

from artella.core import qtutils

logger = logging.getLogger('artella')


class ArtellaException(Exception):
    def __init__(self, message, title='Artella - Error', *args):

        artella_message = 'Artella >>> {}'.format(message)
        logger.error(artella_message)

        super(ArtellaException, self).__init__(artella_message, *args)

        qtutils.show_error_message_box(title, message)


class ArtellaDriveNotAvailable(ArtellaException):
    def __init__(self):
        super(ArtellaDriveNotAvailable, self).__init__(
            'Local Artella Drive is not available. Please launch Artella Drive App.', 'Artella Drive App not available'
        )


class RemoteSessionsNotAvailable(ArtellaException):
    def __init__(self):
        super(RemoteSessionsNotAvailable, self).__init__(
            'No remote sessions available. Please visit your Project Drive in Artella Web App and try again!'
        )


class BadRemoteResponse(ArtellaException):
    def __init__(self, message):
        super(BadRemoteResponse, self).__init__(message)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella API implementation
"""

from __future__ import print_function, division, absolute_import

import artella


def _get_client():
    """
    Returns current Artella Drive Client being used by Artella DCC plugin
    :return: Instance of current Artella Drive Client being used
    :rtype: ArtellaDriveClient
    """

    artella_drive_client = artella.DccPlugin().get_client()
    if not artella_drive_client:
        return None

    return artella_drive_client


def ping():
    """
    Test call that returns whether the Artella Drive is valid or not

    :return: Returns a success response or auth failure message
    :rtype: dict
    :example:
    >>> self.ping()
    {
        message: 'OK';
        response: true;
        status_code: 200;
    }
    """

    client = _get_client()
    return client.ping()


def get_local_root():
    """
    Returns the local storage root path for this machine by asking to remote server.
    .. note::
        If local root cannot be retrieved from server (this can happen because server is down or because user
        doesn't have a working internet connection) then this function will try to retrieve the local root path
        from ARTELLA_FOLDER_ROOT environment variable. This environment variable must be defined by users manually
        to point to local root path.

    :return: Absolute path where files are stored locally
    :rtype: str or None
    :example:
    >>> self.get_local_root()
    "C:/Users/artella/artella-files"
    """

    client = _get_client()
    return client.get_local_root()


def get_local_projects():
    """
    Returns all available project files found in the local user machine

    :return: List of dictionaries containing all the information of the current available
    Artella on local user machine projects
    :rtype: dict(str, dict(str, str))
    """

    client = _get_client()
    return client.get_local_projects()


def download_file(file_path, show_dialogs=True):
    """
    Downloads a file from Artella server

    :param str file_path: File we want to download from Artella server
    :param bool show_dialogs: Whether UI dialogs should appear or not.
    :return: True if the download operation was successful; False otherwise.
    :example:
    >>> self.download_file("C:/Users/artella/artella-files/ProjectA/Assets/test/model/hello.a")
    True
    """

    return artella.DccPlugin().download_file(file_path=file_path, show_dialogs=show_dialogs)





if __name__ == '__main__':
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella register functionality
"""

from __future__ import print_function, division, absolute_import

import artella

# =================================================================================

REGISTER_ATTR = '_registered_classes'

# =================================================================================


def register_class(cls_name, cls, is_unique=True):
    """
    Registers given class in artella module dictionary.

    :param cls_name: str, name of the class we want to register
    :param cls: class, class we want to register
    :param is_unique: bool, Whether if the class should be updated if new class is registered with the same name
    """

    if REGISTER_ATTR not in artella.__dict__:
        artella.__dict__[REGISTER_ATTR] = list()

    if not is_unique and cls_name in artella.__dict__:
        return

    artella.__dict__[cls_name] = cls
    artella.__dict__[REGISTER_ATTR].append(cls_name)


def cleanup():

    if REGISTER_ATTR not in artella.__dict__:
        return

    for cls_name in artella.__dict__[REGISTER_ATTR]:
        if cls_name not in artella.__dict__:
            continue
        del artella.__dict__[cls_name]
    del artella.__dict__[REGISTER_ATTR]



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract scene file parser implementation
"""

from __future__ import print_function, division, absolute_import

from artella import register
from artella.core.dcc import reroute
from artella.core.utils import abstract


class AbstractSceneParser(object):
    """
    Class that defines basic scene parser abstract functions
    """

    @abstract
    @reroute
    def parse(self, file_path=None):
        """
        Parses all the contents of the given file path looking for file paths

        :param str or None file_path: Absolute local file path of the DCC file we want to parse. If not given,
            current opened DCC scene file path will be used
        :return:
        """

        pass

    @abstract
    @reroute
    def update_paths(self, file_path=None):
        """
        Converts all file path of the given DCC file to make sure they point to valid Artella file paths

        :param str or None file_path: Absolute local file path of the DCC file we want to parse. If not given,
            current opened DCC scene file path will be used
        :return:
        """

        pass


register.register_class('Parser', AbstractSceneParser)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for artella
"""

from __future__ import print_function, division, absolute_import

import os
import sys
import logging

import artella
from artella import logger as logger
import artella.register as register
from artella.core import consts, utils


def init(
        init_client=True, plugin_paths=None, dcc_paths=None, extensions=None, dev=False,
        load_plugins=True, create_menu=True, create_callbacks=True):
    """
    Initializes Artella Plugin

    :param bool init_client: Whether or not Artella Drive Client should be initialized during initialization.
        Useful to avoid to connect to Artella client when developing DCC specific functionality.
    :param list(str) plugin_paths: List of paths where Artella plugins can be located
    :param list(str) dcc_paths: List of paths where Artella DCC implementations can be located
    :param list(str) extensions: List of extensions to register
    :param bool dev: Whether or not initialization should be done in dev mode
    :param bool load_plugins: Whether or not Artella Plugins should be loaded
    :param bool create_menu: Whether or not Artella menu should be created
    :param bool create_callbacks: Whether or not Artella DCC plugin callbacks should be created
    :return: True if Artella initialization was successful; False otherwise.
    :rtype: bool
    """

    plugins_path = plugin_paths if plugin_paths is not None else list()
    dccs_path = dcc_paths if dcc_paths is not None else list()
    extensions = extensions if extensions is not None else list()

    # Create logger
    logger.create_logger()
    artella_logger = logging.getLogger('artella')

    # Register DCC paths
    valid_dcc_paths = list()
    default_dccs_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'dccs')
    if default_dccs_path not in dccs_path:
        dccs_path.append(default_dccs_path)
    for dcc_path in dccs_path:
        if os.path.isdir(dcc_path):
            if dcc_path not in sys.path:
                sys.path.append(dcc_path)
            valid_dcc_paths.append(dcc_path)
    if valid_dcc_paths:
        if os.environ.get(consts.AED, None):
            env = os.environ[consts.AED].split(';')
            env.extend(valid_dcc_paths)
            clean_env = list(set([utils.clean_path(pth) for pth in env]))
            dcc_paths_str = ';'.join(clean_env)
        else:
            dcc_paths_str = ';'.join(valid_dcc_paths)
    if os.environ.get(consts.AED, ''):
        if dcc_paths_str:
            os.environ[consts.AED] += ';{}'.format(dcc_paths_str)
    else:
        os.environ[consts.AED] = dcc_paths_str

    # Once DCC paths are registered we can import modules
    import artella
    import artella.register as register

    import artella.dcc as dcc

    # Do not remove this imports, they do registration of some of the core Artella classes
    from artella.core import dcc as dcc_core
    from artella.core import client, resource, plugin, dccplugin, qtutils
    from artella.widgets import theme, color

    # Make sure that Artella Drive client and DCC are cached during initialization
    current_dcc = dcc_core.current_dcc()
    if not current_dcc:
        artella_logger.error('Impossible to load Artella Plugin because no DCC is available!')
        return False

    # Specific DCC extensions are managed by the client
    dcc_extensions = dcc.extensions() or list()
    extensions.extend(dcc_extensions)

    # Initialize resources and theme
    resources_mgr = artella.ResourcesMgr()
    resources_mgr.register_resources_path(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'resources'))
    if qtutils.QT_AVAILABLE:
        artella_theme = theme.ArtellaTheme(main_color=color.ArtellaColors.DEFAULT)
        register.register_class('theme', artella_theme)

    # Create Artella Drive Client
    artella_drive_client = client.ArtellaDriveClient.get(extensions=extensions) if init_client else None

    # Load Plugins
    if load_plugins:
        default_plugins_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'plugins')
        if default_plugins_path not in plugins_path:
            plugins_path.append(default_plugins_path)
        artella.PluginsMgr().register_paths(plugins_path)
        artella.PluginsMgr().load_registered_plugins()

    # Initialize Artella DCC plugin
    artella.DccPlugin(artella_drive_client).init(
        dev=dev, show_dialogs=False, create_menu=create_menu, create_callbacks=create_callbacks)

    return True


def shutdown(dev=False, cleanup_modules=False):
    """
    Shutdown Artella Plugin

    :param bool cleanup_modules: Whether import artella modules should be removed from sys.path or not
    :return: True if Artella shutdown was successful; False otherwise.
    :rtype: bool
    """

    # Create logger
    logger.create_logger()

    try:
        artella.PluginsMgr().shutdown(dev=dev)
        artella.DccPlugin().shutdown(dev=dev)
    except Exception as exc:
        pass

    # Cleanup artella modules
    if cleanup_modules:

        register.cleanup()

        for k in sys.modules.keys():
            if k.startswith('artella.'):
                del sys.modules[k]

        global CURRENT_DCC
        CURRENT_DCC = None

    return True



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract menu functions
"""

from artella.core.dcc import reroute
from artella.core.utils import abstract


@reroute
@abstract
def main_menu_toolbar():
    """
    Returns Main menu toolbar where DCC menus are created by default

    :return: Native object that represents main menu toolbar in current DCC
    :rtype: object
    """

    pass


@reroute
@abstract
def get_menus():
    """
    Return all the available menus in current DCC. This function returns specific DCC objects that represents DCC
    UI menus.

    :return: List of all menus names in current DCC
    :rtype: list(str)
    """

    pass


@reroute
@abstract
def get_menu_items():
    """
    Returns all available menu items in current DCC. This function returns specific DCC objects that represents DCC
    UI menu items.

    :return: List of all menu item names in current DCC
    :rtype: list(str)
    """

    pass


@reroute
@abstract
def get_menu(menu_name):
    """
    Returns native DCC menu with given name
    :param str menu_name: name of the menu to search for
    :return: Native DCC menu object or None if the menu does not exists
    :rtype: str or None
    """

    pass


@reroute
@abstract
def get_menu_item(menu_item_name):
    """
    Returns native DCC menu item with given name
    :param str menu_item_name: name of the menu item to search for
    :return: Native DCC menu object or None if the menu does not exists
    :rtype: str or None
    """

    pass


@reroute
@abstract
def check_menu_exists(menu_name):
    """
    Returns whether or not menu with given name exists

    :param str menu_name: name of the menu to search for
    :return: True if the menu already exists; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def add_menu(menu_name, parent_menu=None, tear_off=True, icon='', **kwargs):
    """
    Creates a new DCC menu.

    :param str menu_name: name of the menu to create
    :param object parent_menu: parent menu to attach this menu into. If not given, menu will be added to
    specific DCC main menu toolbar. Must be specific menu DCC native object
    :param bool tear_off: Whether or not new created menu can be tear off from its parent or not
    :param str icon: name of the icon to be used in this menu
    :return: New DCC native menu object created or None if the menu item was not created successfully
    :rtype: object or None
    """

    pass


@reroute
@abstract
def remove_menu(menu_name):
    """
    Removes menu from current DCC if exists

    :param str menu_name: name of the menu to remove
    :return: True if the removal was successful; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def add_menu_item(menu_item_name, menu_item_command='', parent_menu=None, icon='', **kwargs):
    """
    Adds a new menu item to the given parent menu. When the item is clicked by the user the given command will be+
    executed
    :param str menu_item_name: name of the menu item to create
    :param str menu_item_command: command to execute when menu item is clicked
    :param object parent_menu: parent menu to attach this menu into. Must be specific menu DCC native object
    :param str icon: name of the icon to be used in this menu item
    :return: New DCC native menu item object created or None if the menu item was not created successfully
    :rtype: object or None
    """

    pass


@reroute
@abstract
def add_sub_menu_item(menu_item_name, menu_item_command='', parent_menu=None, icon='', **kwargs):
    """
    Adds a new sub menu item to the given parent menu. When the item is clicked by the user the given command will be+
    executed
    :param str menu_item_name: name of the menu item to create
    :param str menu_item_command: command to execute when menu item is clicked
    :param object parent_menu: parent menu to attach this menu into. Must be specific menu DCC native object
    :param str icon: name of the icon to be used in this menu item
    :return: New DCC native menu item object created or None if the menu item was not created successfully
    :rtype: object or None
    """

    pass


@reroute
@abstract
def remove_menu_item(menu_item_name, parent_menu):
    """
    Removes a menu item from the given parent menu.
    :param str menu_item_name: name of the menu item to remove
    :param object parent_menu: parent menu to remove this menu from. Must be specific menu DCC native object
    :return: Trie if the operation was successful; False otherwise.
    :rtype: bool
    """

    pass


@reroute
@abstract
def add_menu_separator(parent_menu):
    """
    Adds a new separator to the given parent menu
    :param object parent_menu: parent menu to attach this menu into. Must be specific menu DCC native object
    """

    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract window implementation
"""

from __future__ import print_function, division, absolute_import

import artella
from artella import register
from artella.core import qtutils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets

if not qtutils.QT_AVAILABLE:
    class AbstractWindow(object):
        pass
else:
    class AbstractWindow(QtWidgets.QMainWindow):
        def __init__(self, parent=None, **kwargs):

            closed = QtCore.Signal()

            if not parent:
                from artella import dcc
                parent = dcc.get_main_window()

            super(AbstractWindow, self).__init__(parent, **kwargs)

            self._pos_anim = QtCore.QPropertyAnimation(self)
            self._pos_anim.setTargetObject(self)
            self._pos_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
            self._pos_anim.setDuration(300)
            self._pos_anim.setPropertyName('pos')

            self._opacity_anim = QtCore.QPropertyAnimation()
            self._opacity_anim.setTargetObject(self)
            self._opacity_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
            self._opacity_anim.setDuration(300)
            self._opacity_anim.setPropertyName('windowOpacity')
            self._opacity_anim.setStartValue(0.0)
            self._opacity_anim.setEndValue(1.0)

            self.setup_ui()

            self._fade_in()

        def get_main_layout(self):
            main_layout = QtWidgets.QVBoxLayout()
            main_layout.setContentsMargins(0, 0, 0, 0)
            main_layout.setSpacing(0)

            return main_layout

        def setup_ui(self):
            main_widget = QtWidgets.QWidget()
            self.main_layout = self.get_main_layout()
            main_widget.setLayout(self.main_layout)
            self.setCentralWidget(main_widget)

            artella_frame = QtWidgets.QFrame()
            artella_frame_layout = QtWidgets.QHBoxLayout()
            artella_frame.setLayout(artella_frame_layout)
            artella_frame.setStyleSheet('background: rgb(23, 165, 151)')

            artella_header = QtWidgets.QLabel()
            artella_header_pixmap = artella.ResourcesMgr().pixmap('artella_header')
            artella_header.setPixmap(artella_header_pixmap)
            artella_frame_layout.addStretch()
            artella_frame_layout.addWidget(artella_header)
            artella_frame_layout.addStretch()

            self.main_layout.addWidget(artella_frame)

        def fade_close(self):
            self._fade_out()

        # =================================================================================================================
        # INTERNAL
        # =================================================================================================================

        def _fade_out(self):
            self._opacity_anim.finished.connect(self.close)
            self._pos_anim.setDirection(QtCore.QAbstractAnimation.Backward)
            self._pos_anim.start()
            self._opacity_anim.setDirection(QtCore.QAbstractAnimation.Backward)
            self._opacity_anim.start()

        def _fade_in(self):
            self._pos_anim.start()
            self._opacity_anim.start()


register.register_class('Window', AbstractWindow)
